-------------------------------------------------------------------------
--
--  MR4 PROCESSOR   -  25/october/2005
--              Ney Calazans / Fernando Moraes
--
--
--  NOV/2007  - VERSION WITH DATA AND INSTRUCTION MEMORIES
-------------------------------------------------------------------------
library IEEE;
use IEEE.Std_Logic_1164.all;

package p_MR4 is

	subtype reg32  is std_logic_vector(31 downto 0);
	
	-- inst_type defines the instructions decodeable by the control unit
	type inst_type is
	(
		AAND,
		ADDIU,
		ADDU,
		ANDI,
		BEQ,
		BGEZ,
		BLEZ,
		BLTZ,
		BNE,
		DIVU,
		J,
		JAL,
		JALR,
		JR,
		LBU,
		LUI,
		LW,
		MFHI,
		MFLO,
		MULTU,
		NNOR,
		OOR,
		ORI,
		SB,
		SLLV,
		SLT,
		SLTI,
		SLTIU,
		SLTU,
		SRAV,
		SRLV,
		SSLL,
		SSRA,
		SSRL,
		SUBU,
		SW,
		XORI,
		XXOR,
		invalid_instruction
	);
	
	type microinstruction is record
		CY1:      std_logic;   -- command of the first stage
		CY2:      std_logic;   -- command of the second stage
		walu:     std_logic;   -- command of the third stage
		wmdr:     std_logic;   -- command of the fourth stage
		wpc:      std_logic;   -- PC write enable
		wreg:     std_logic;   -- register bank write enable
		ce:       std_logic;   -- Chip enable and R_W controls
		rw:       std_logic;
		bw:       std_logic;   -- Byte-word control (mem write only)
		i:        inst_type;   -- operation specification
		init_mul: std_logic;   -- command to start multiplication
		init_div: std_logic;   -- command to start division
	end record;

end p_MR4;
